from fp_xls_2_xml import xls2xml, xml2xls
from fp_xls_2_xml.convertors import XLSConverter, XMLConverter
from pathlib import Path
import shutil
import tempfile


TEST_FOLDER = Path(__file__).parent
XLS = TEST_FOLDER / 'example.xlsx'
XML = TEST_FOLDER / 'example.xml'
XMLUTF8 = TEST_FOLDER / 'exampleUTF8.xml'
TEMP = tempfile.TemporaryDirectory()
TEMP_PATH = Path(TEMP.name)


def test_xls_convertor():
    xls_conv = XLSConverter(XLS)
    data_source = xls_conv.data
    xls_conv.serialize(TEMP_PATH / 'test.xlsx')
    xls_conv.deserialize(TEMP_PATH / 'test.xlsx')
    data_result = xls_conv.data
    assert data_result.to_csv() == data_source.to_csv()

def test_xml_convertor():
    xml_conv = XMLConverter(XML)
    data_source = xml_conv.data
    xml_conv.serialize(save_path=TEMP_PATH / 'test.xml')
    xml_conv.deserialize(load_path=TEMP_PATH / 'test.xml')
    data_result = xml_conv.data
    xml_source = data_source.toxml().replace('\n', '').replace('\t', '')
    xml_result = data_result.toxml().replace('\n', '').replace('\t', '')
    assert xml_source == xml_result

def test_xmlUTF8_convertor():
    xml_conv = XMLConverter(XMLUTF8)
    data_source = xml_conv.data
    xml_conv.serialize(save_path=TEMP_PATH / 'test.xml')
    xml_conv.deserialize(load_path=TEMP_PATH / 'test.xml')
    data_result = xml_conv.data
    xml_source = data_source.toxml().replace('\n', '').replace('\t', '')
    xml_result = data_result.toxml().replace('\n', '').replace('\t', '')
    assert xml_source == xml_result


def test_final():
    xls_source = TEMP_PATH / 'test.xls'
    xml_source = TEMP_PATH / 'test.xml'
    xls_result = TEMP_PATH / 'result.xls'
    xml_result = TEMP_PATH / 'result.xml'
    shutil.copy(XLS, xls_source)
    shutil.copy(XML, xml_source)
    xls2xml(xls_source, xml_result)
    xml2xls(xml_result, xls_result)



import shutil
from pathlib import Path

from fp_xls_2_xml import xls2xml, xml2xls

from kivy.app import App
from kivy.lang import Builder
from kivy.uix.boxlayout import BoxLayout
from kivy.uix.label import Label
from kivy.uix.popup import Popup
from kivy.core.window import Window


Builder.load_string('''
<Tair>:
    TextInput:
        text: 'drag and drop the file into this window'
        font_size: 30
        padding_x:
            [self.center[0] - self._get_text_width(max(self._lines, key=len), self.tab_width, self._label_cached) / 2.0,
            0] if self.text else [self.center[0], 0]
        padding_y: [self.height / 2.0 - (self.line_height / 2.0) * len(self._lines), 0]
''')
class Tair(BoxLayout):
    pass


class FPXLS2XML(App):
    def build(self):
        Window.bind(on_dropfile=self._on_file_drop)
        self.tair = Tair()
        return self.tair

    def _on_file_drop(self, window, file_path):
        file = Path(file_path.decode())
        try:
            if "xls" in file.name[-5:]:
                out = file.parent / "out.xml"
                xls2xml(file, out)
            else:
                out = file.parent / "out.xls"
                xml2xls(file, out)
            content = Label(text='Saved to: {}'.format(out))
            popup = Popup(title="Success!", content=content, auto_dismiss=False)
            content.bind(on_touch_down=popup.dismiss)
            popup.open()
        except Exception:
            content = Label(text='Failed to save: {}'.format(file))
            popup = Popup(title="Failed!", content=content, auto_dismiss=False)
            content.bind(on_touch_down=popup.dismiss)
            popup.open()


if __name__ == '__main__':
    FPXLS2XML().run()


import time


def make_orders_list(xml, orders):
    for order in orders:
        make_order(xml, order)

def make_order(xml, order, i=1):
    element_root, *_ = xml.data.getElementsByTagName("OrderList")

    element_order = xml.data.createElement("Order")
    ext_ref = int((time.time() * pow(10, 7)) / 1.20000103)
    element_order.setAttribute("ext_ref", str(ext_ref))
    element_order.setAttribute("presc", "2")

    element_order_cust = xml.data.createElement("Cust")
    element_order_cust.setAttribute("code", order["cust_code"])
    element_order.appendChild(element_order_cust)

    element_order_information = xml.data.createElement("OrdInfo")
    element_order_information.setAttribute("priority", order["order_priority"])
    element_order.appendChild(element_order_information)

    element_order_instruction = xml.data.createElement("OrdInst")
    inner = xml.data.createTextNode(order["order_intructions"])
    element_order_instruction.appendChild(inner)
    element_order.appendChild(element_order_instruction)

    full_count = int(order["count"])
    for batch_index in range((full_count // 15) + 1):
        count = full_count - (batch_index * 15)
        count = count if count < 15 else 15
        if count:
            make_row(xml, order, str(count), element_order)

    element_root.appendChild(element_order)


def make_row(xml, order, count, element_order):
    order_rows_count = len(element_order.getElementsByTagName("Row"))

    element_row = xml.data.createElement("Row")
    element_row.setAttribute("code", str(order_rows_count + 1))
    element_row.setAttribute("qty", count)

    element_row_prod = xml.data.createElement("Prod")
    element_row_prod.setAttribute("code", order["product_code"])
    element_row.appendChild(element_row_prod)

    element_row_material = xml.data.createElement("Mat")
    element_row_material.setAttribute("code", order["material_code"])
    element_row.appendChild(element_row_material)

    element_row_color = xml.data.createElement("Col")
    element_row_color.setAttribute("code", order["color_code"])
    element_row.appendChild(element_row_color)

    element_parlist = xml.data.createElement("ParList")
    element_parlist.setAttribute("pwr", order["parlist_pwr"])

    for param in ["001", "009", "400", "401", "403", "404"]:
        element_par = xml.data.createElement("Par")
        key = "codes_{}".format(param)
        element_par.setAttribute("code", param)
        element_par.setAttribute("val", order[key])
        element_parlist.appendChild(element_par)

    element_row.appendChild(element_parlist)

    if "tech_instructions" in order:
        element_tech_instructions = xml.data.createElement("TechInst")
        inner = xml.data.createTextNode(order["tech_instructions"])
        element_tech_instructions.appendChild(inner)
        element_row.appendChild(element_tech_instructions)

    if "fit_notes" in order:
        element_fit_notes = xml.data.createElement("FitNotes")
        inner = xml.data.createTextNode("Fit 0.5D steeper")
        element_fit_notes.appendChild(inner)
        element_row.appendChild(element_fit_notes)

    if "barcode" in order:
        element_srv_list = xml.data.createElement("SrvList")

        element_par = xml.data.createElement("Srv")
        element_par.setAttribute("code", order["barcode"])
        element_srv_list.appendChild(element_par)

        element_row.appendChild(element_srv_list)

    element_order.appendChild(element_row)



from pathlib import Path
from fp_xls_2_xml.fpxls2xml import xml2xls, xls2xml
from fire import Fire


def enter(input, output):
    input_path = Path(input)
    output_path = Path(output)
    if "xls" in input_path.name[-5:]:
        xls2xml(input_path, output_path)
    else:
        xml2xls(input_path, output_path)


def main():
    Fire(enter)


if __name__ == '__main__':
    main()


from pathlib import Path
from copy import deepcopy
from xml.dom.minidom import parseString
from xml.parsers.expat import ExpatError
import pandas


class XMLConverter:
    def __init__(self, convertable=None):
        if not convertable:
            self.data = parseString(
                '<OrderList />'
            )
        elif isinstance(convertable, dict):
            self.data = deepcopy(convertable)
        else:
            self.deserialize(load_path=convertable)
    
    def __repr__(self):
        return self.data.toprettyxml()

    def serialize(self, save_path=None):
        xml_string = self.data.toprettyxml(encoding="utf-16")
        if save_path:
            with open(save_path, 'wb') as file:
                file.write(xml_string)
        return xml_string

    def deserialize(self, xml_string=None, load_path=None):
        if load_path:
            with open(load_path, 'rb') as file:
                xml_string = file.read()
        try:
            self.data = parseString(xml_string.decode("utf-16"))
        except ExpatError:
            self.data = parseString(xml_string.decode("utf-8"))


class XLSConverter:
    def __init__(self, load_path=None, data=None):
        self.data = data if hasattr(data, "empty") else pandas.DataFrame()
        if load_path:
            self.deserialize(load_path)

    def serialize(self, save_path):
        self.data.to_excel(save_path, index=False)

    def deserialize(self, load_path):
        with open(load_path, "rb") as file:
            self.data = pandas.read_excel(file)



from setuptools import setup, find_packages
from string import ascii_letters


with open("requirements.txt") as f:
    requirements = [l.strip() for l in f.readlines() if l[0] in ascii_letters]


setup(
    name='fp_xls_2_xml',
    version='1.0.1',
    packages=["fp_xls_2_xml"],
    package_dir={'fp_xls_2_xml': 'fp_xls_2_xml'},
    package_data={'fp_xls_2_xml': ['*.kv']},
    install_requires=requirements,
    python_requires='>=3.5.*',
    entry_points={
        'console_scripts': [
            'fpxls2xml=fp_xls_2_xml.cli:main',
        ],
    },
)



import pandas
from .processors import make_orders_list
from .convertors import XMLConverter, XLSConverter

SHEET_KEY_TO_DATA_KEY = {
    'Номер заказа': 'order',
    'Код клиента': 'cust_code',
    'Приоритет': 'order_priority',
    'Параметры линзы одной строкой': 'order_intructions',
    'Количество': 'count',
    'Design': 'product_code',
    'Мат-л': 'material_code',
    'Цвет': 'color_code',
    'PWR': 'parlist_pwr',
    'Diam': 'codes_001',
    'BC': 'codes_009',
    'RCD/RCD_flat': 'codes_400',
    'AZA/AZA_flat': 'codes_401',
    'RCD_steep': 'codes_404',
    'AZA_steep': 'codes_403'
}
# 'Статус трансфера', 'Номер в наборе', 'Cерийный номер'
# tech_instructions, fit_notes
DATA_KEYS = [
    "order",
    "cust_code",
    "order_priority",
    "order_intructions",
    "count",
    "product_code",
    "parlist_pwr",
    "codes_009",
    "codes_400",
    "codes_401",
    "codes_403",
    "codes_404",
    "codes_001",
    "material_code",
    "color_code",
    "tech_instructions",
    "barcode"
]


def order_from_table_row(row, keys):
    data = {}
    for k, key in SHEET_KEY_TO_DATA_KEY.items():
        data[key] = str(row[k])
    #for i, k in enumerate(DATA_KEYS):
    #    data[k] = str(row[keys[i]])
    print(data)
    return data


def xls2xml(path, destination):
    xls = XLSConverter(path)
    keys = [k for k in xls.data]
    print(keys)
    orders = []
    for _, row in xls.data.iterrows():
        orders.append(order_from_table_row(row, keys))
    xml = XMLConverter()
    make_orders_list(xml, orders)
    xml.serialize(destination)


def xml2xls(path, destination):
    xml = XMLConverter(path)
    elements_orders = xml.data.getElementsByTagName("Order")
    orders = []
    for i, element_order in enumerate(elements_orders):
        order = dict(zip(DATA_KEYS, ["Error"]*len(DATA_KEYS)))
        order["order"] = i
        element_cust_code, *_ = element_order.getElementsByTagName("Cust")
        order["cust_code"] = element_cust_code.getAttribute("code")
        element_order_priority, *_ = element_order.getElementsByTagName("OrdInfo")
        order["order_priority"] = element_order_priority.getAttribute("priority")
        element_order_intructions, *_ = element_order.getElementsByTagName("OrdInst")
        inner, *_ = element_order_intructions.childNodes
        order["order_intructions"] = inner.data
        rows = {
            "count": 0,
            "product_code": "",
            "material_code": "",
            "color_code": "",
        }
        for row in element_order.getElementsByTagName("Row"):
            rows["count"] += int(row.getAttribute('qty'))
            element_product_code, *_ = element_order.getElementsByTagName("Prod")
            rows["product_code"] = element_product_code.getAttribute("code")
            element_material_code, *_ = element_order.getElementsByTagName("Mat")
            rows["material_code"] = element_material_code.getAttribute("code")
            element_color_code, *_ = element_order.getElementsByTagName("Col")
            rows["color_code"] = element_color_code.getAttribute("code")
            element_parlist, *_ = element_order.getElementsByTagName("ParList")
            rows["parlist_pwr"] = element_parlist.getAttribute("pwr")
            for par in element_parlist.getElementsByTagName("Par"):
                code = par.getAttribute("code")
                val = par.getAttribute("val")
                order["codes_{}".format(code)] = val
        order.update(rows)
        element_tech_instructions, *_ = element_order.getElementsByTagName("TechInst")
        inner, *_ = element_tech_instructions.childNodes
        order["tech_instructions"] = inner.data
        order['barcode'] = "12"
        order['order'] = str(order['order'])
        order['count'] = str(order['count'])
        orders.append(order.values())
    xls = XLSConverter(data=pandas.DataFrame(data=orders, columns=DATA_KEYS))
    xls.serialize(destination)
